//---------------------------------------------------------------------------
// Klasse     : TCP
// Datei      : TCP.cpp
// Datum      : 11.07.2007
// Version    : 1.0
// Autor      : Karsten Donat
//---------------------------------------------------------------------------

#include "TCP.h"

#include "Stack.h"
#include "ENC28J60.h"
#include "uart.h"
#include "Uhr.h"
#include <String.h>
#include <avr/pgmspace.h>

TCP_Socket TCP_Sockets[TCP_Socket_Anzahl];

void TCP_Leere_Socket(Byte SocketID);


int TCP_Create()
{
   // Initialisierungsroutine

   // Sockets leeren
   for (Byte b=0; b<TCP_Socket_Anzahl; b++)
      TCP_Leere_Socket(b);

   return 0;
}

int TCP_Destroy()
{
   // ToDo: Aufrumen
   return 0;
}

// leert einen Socket
void TCP_Leere_Socket(Byte SocketID)
{
   TCP_Sockets[SocketID].Client_IP    = 0;
   TCP_Sockets[SocketID].Client_Port  = 0; // Port der Anwendung auf den anfragenden Client
   TCP_Sockets[SocketID].Server_Port  = 0; // Port der Anwendung auf dem ATMega
   TCP_Sockets[SocketID].Last_Tick    = 0; // letzter Tickwert um Zeitdauer seit letztem Austausch zu berechnen
   TCP_Sockets[SocketID].Phase        = 0;
}

// sucht einen TCP Socket, ist dieser nicht vorhanden wird ein neuer angelegt und die ID zurckgegeben
// steht kein freier Socket zur Verfgung, wird 0 zurckgegeben
//Byte TCP_Suche_Socket(Byte Client_IP[4], Word Client_Port, Word Server_Port)
Byte TCP_Suche_Socket(DWord Client_IP, Word Client_Port, Word Server_Port)
{
   Byte Leerer_Socket = TCP_Socket_Anzahl;
   Byte b=0;
   // Socket suchen, dabei leeren Socket merken
   while ((b<TCP_Socket_Anzahl) and
      ( (       Client_IP   !=        TCP_Sockets[b].Client_IP  ) or
        (       Client_Port !=        TCP_Sockets[b].Client_Port) or
        (       Server_Port !=        TCP_Sockets[b].Server_Port) ) )
   {
      if (TCP_Sockets[b].Client_Port==0)
         Leerer_Socket = b;
      b++;
   }

   // gltiger Socket gefunden
   if (b<TCP_Socket_Anzahl)
      return b;
   else
   // leerer Socket -> belegen
   if (Leerer_Socket<TCP_Socket_Anzahl)
   {
      TCP_Sockets[Leerer_Socket].Client_IP    = Client_IP;

      TCP_Sockets[Leerer_Socket].Client_Port  = Client_Port; // Port der Anwendung auf den anfragenden Client
      TCP_Sockets[Leerer_Socket].Server_Port  = Server_Port; // Port der Anwendung auf dem ATMega
      TCP_Sockets[Leerer_Socket].Last_Tick    = Ticks;       // letzter Tickwert um Zeitdauer seit letztem Austausch zu berechnen
      TCP_Sockets[Leerer_Socket].Phase        = 0;
      return Leerer_Socket;
   }
   // Socket nicht vorhanden und kein Platz einen neuen anzulegen
   else
      return 255;
}

void TCP_Verarbeite_Packet(Word len, Byte *buff)
{
   {uart_puts("BeginA");
         // empfangenes Datenpacket per RS232 zurck senden
         Byte* c = (Byte*)buffer;
//         for (Word i=0;i<packet_lenght;i++)
         for (Word i=0;i<len;i++)
         {
            uart_putc(*c++);
         }
   uart_puts("EndeA");}


   struct TCP_Header *TCP;
   TCP = (struct TCP_Header *)&buff[TCP_Begin]; // TCP Packet (hinter Eth.+IP Paketen)
   Byte SocketID = TCP_Suche_Socket(
      ((DWord)buffer[26]<<24)+((DWord)buffer[27]<<16)+((DWord)buffer[28]<<8)+(DWord)buffer[29],
      (TCP->Source_Port_Hi     <<8) + TCP->Source_Port_Lo,
      (TCP->Destination_Port_Hi<<8) + TCP->Destination_Port_Lo);   // Socket zur aktuellen TCP Verbindung im Puffer suchen

   Word Datenlaenge_Empfangen = len - (ETH_HEADERLENGHT +IP_HEADERLENGHT +(TCP->DataOffset.Val <<2) ) - 4;
   Word Datenlaenge_Senden    = 0;

   // kein freier Socket
   if (SocketID == 255)
      return;

   TCP_Sockets[SocketID].Last_Tick    = Ticks;       // letzter Tickwert um Zeitdauer seit letztem Austausch zu berechnen


   //An den Port zurcksenden, von dem das Packet gekommen ist
   Byte TempPort;
   TempPort = TCP->Source_Port_Hi;
   TCP->Source_Port_Hi = TCP->Destination_Port_Hi;
   TCP->Destination_Port_Hi = TempPort;
   TempPort = TCP->Source_Port_Lo;
   TCP->Source_Port_Lo = TCP->Destination_Port_Lo;
   TCP->Destination_Port_Lo = TempPort;

   // SQE und ACK Nummern austauschen, dabei alte Nummer aus SEQ um 1 erhhen
   // Nummer aus SEQ nach ACK kopieren und um 1 erhhen
   DWord SEQalt;
   DWord ACKalt;

   SEQalt = ((DWord)TCP->Sequence_Number_B3      <<24) + ((DWord)TCP->Sequence_Number_B2      <<16) + ((DWord)TCP->Sequence_Number_B1      <<8) + (DWord)TCP->Sequence_Number_B0;
   ACKalt = ((DWord)TCP->Acknowlegement_Number_B3<<24) + ((DWord)TCP->Acknowlegement_Number_B2<<16) + ((DWord)TCP->Acknowlegement_Number_B1<<8) + (DWord)TCP->Acknowlegement_Number_B0;

   // Verbindungsversuch vom Client
   if ((TCP_Sockets[SocketID].Phase == 0)and
       (TCP->Flags.bits.SYN==1)and
       (TCP->Flags.bits.ACK==0))
   {
      TCP->Flags.bits.ACK = 1; // Verbindungsaufbau akzeptieren

      SEQalt++;
      ACKalt = Ticks; // eigene SEQ Nummer generieren

      TCP_Sockets[SocketID].Phase = 2; // Phase 1 ist implizit
   }

   if ((TCP_Sockets[SocketID].Phase == 2)and
       (TCP->Flags.bits.SYN==0)and
       (TCP->Flags.bits.ACK==1))
   {
      TCP_Sockets[SocketID].Phase = 3;
      return; // nichts tun. war nur Besttigung
   }

   // Client sendet Daten
   if ((TCP_Sockets[SocketID].Phase == 3)and
       (TCP->Flags.bits.SYN==0)and
       (TCP->Flags.bits.PSH==1)and
       (TCP->Flags.bits.ACK==1))
   {
      // Rckgabe in Puffer schreiben

      // HTML Header
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("HTTP/1.1 200 OK"),Terminate_Windows);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("Server: WHZ Pendel (Atmel)"),Terminate_Windows);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR(""),Terminate_Windows);

      // HTML Code
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("<html><body><b>Pendel</b><br><br>Geschafft! "),DONT_TERMINATE);
      DateTimeToStr((char*) &buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden]);Datenlaenge_Senden += 19;

      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("<br>Freier RAM: "),DONT_TERMINATE);
      IntToStr(momentan_frei_,(char*) &buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden]);
      //if (momentan_frei_>=10000) Datenlaenge_Senden += 5;
      //if (momentan_frei_>= 1000) Datenlaenge_Senden += 4;
      if (momentan_frei_>=  100) Datenlaenge_Senden += 3;
      else if (momentan_frei_>=   10) Datenlaenge_Senden += 2;
      else if (momentan_frei_<    10) Datenlaenge_Senden += 1;
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("</body></html>"),DONT_TERMINATE);

      // index.html

/*      // HTML Header
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Header      ,DONT_TERMINATE);

      // HTML Anfang
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Anfang      ,DONT_TERMINATE);

      // Datum Uhrzeit
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Anfang,DONT_TERMINATE);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("Pendelzeit"),DONT_TERMINATE);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Mitte ,DONT_TERMINATE);
      DateTimeToStr((char*) &buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden]);Datenlaenge_Senden += 19;
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Ende  ,DONT_TERMINATE);

      // freier RAM
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Anfang,DONT_TERMINATE);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) PSTR("freier RAM (Bytes)"),DONT_TERMINATE);
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Mitte ,DONT_TERMINATE);
      IntToStr(momentan_frei_,(char*) &buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden]);
      //if (momentan_frei_>=10000) Datenlaenge_Senden += 5;
      //if (momentan_frei_>= 1000) Datenlaenge_Senden += 4;
      if (momentan_frei_>=  100) Datenlaenge_Senden += 3;
      else if (momentan_frei_>=   10) Datenlaenge_Senden += 2;
      else if (momentan_frei_<    10) Datenlaenge_Senden += 1;
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Zeile_Ende  ,DONT_TERMINATE);

      // HTML Ende
      Datenlaenge_Senden += Print_in_Buffer_Flash_p(&buffer[TCP_Begin+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden], (Byte*) HTML_Ende        ,DONT_TERMINATE);
*/

      SEQalt = SEQalt + Datenlaenge_Empfangen;
      //ACKalt = ACKalt + Datenlaenge_Senden+1;

      // Ende der bertragung
      TCP->Flags.bits.FIN = 1;
      TCP_Sockets[SocketID].Phase = 4;
   }

   // Client quitiert das Ende der Sitzung
   if ((TCP_Sockets[SocketID].Phase == 4)and
       (TCP->Flags.bits.SYN==0)and
       (TCP->Flags.bits.FIN==1)and
       (TCP->Flags.bits.ACK==1))
   {
      TCP_Leere_Socket(SocketID);
      //return;
   }


   Ethernet_II_Header_erzeugen(buff); // ETHERNET II header erzeugen
   buff[16] = ((IP_HEADERLENGHT+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden) & 0xFF00)>>8;
   buff[17] = ((IP_HEADERLENGHT+(TCP->DataOffset.Val <<2)+Datenlaenge_Senden) & 0x00FF); // ip header lenght anpassen
   IP_Header_erzeugen(buff);          // IP header erzeugen

   TCP->Sequence_Number_B3       = (Byte) (ACKalt >> 24) & 0xFF;
   TCP->Sequence_Number_B2       = (Byte) (ACKalt >> 16) & 0xFF;
   TCP->Sequence_Number_B1       = (Byte) (ACKalt >>  8) & 0xFF;
   TCP->Sequence_Number_B0       = (Byte)  ACKalt        & 0xFF;

   TCP->Acknowlegement_Number_B3 = (Byte) (SEQalt >> 24) & 0xFF;
   TCP->Acknowlegement_Number_B2 = (Byte) (SEQalt >> 16) & 0xFF;
   TCP->Acknowlegement_Number_B1 = (Byte) (SEQalt >>  8) & 0xFF;
   TCP->Acknowlegement_Number_B0 = (Byte)  SEQalt        & 0xFF;


   // Checksumme berechnen
   Word sum;
   DWord zsumme;
   TCP->Checksum_Hi = 0x00; // Checksummenfeld liegt im Fokus, darf aber keine Auswirkung haben
   TCP->Checksum_Lo = 0x00;

   // Pseudoheader berechnen
   zsumme =   (buffer[26]<<8) + buffer[27] // SourceAddress
            + (buffer[28]<<8) + buffer[29]
            + (buffer[30]<<8) + buffer[31] // DestAddress
            + (buffer[32]<<8) + buffer[33]
            + 6 // Protocol TCP
            + (TCP->DataOffset.Val <<2) + Datenlaenge_Senden; // TCP Lnge

   sum = checksum(&buff[34],zsumme,( TCP->DataOffset.Val <<2)+Datenlaenge_Senden); // *4 = <<2
sum=sum-2;// Test
   TCP->Checksum_Hi = sum >> 8;
   TCP->Checksum_Lo = sum & 0x00FF;


   // Packet abschicken
   enc28j60PacketSend(ETH_HEADERLENGHT +IP_HEADERLENGHT  +(TCP->DataOffset.Val <<2) + Datenlaenge_Senden,buff);	

}

// index.html

char HTML_Header[] PROGMEM =
   "HTTP/1.1 200 OK" CRLF
   "Server: WHZ Pendel (Atmel)" CRLF
    CRLF;

char HTML_Anfang[] PROGMEM =
   "<html>" CRLF
      "<head>" CRLF
         "<meta http-equiv="AZ"content-type"AZ" "AZ"content="AZ"text/html;charset=utf-8"AZ" />" CRLF
         "<meta name="AZ"generator"AZ" content="AZ"Adobe GoLive"AZ" />" CRLF
         "<title>"Kopf_WHZ_"</title>" CRLF
      "</head>" CRLF
      "<body>" CRLF
         "<font face="AZ"Verdana, Arial, Helvetica, sans-serif"AZ">" CRLF
            "<table width="AZ"100%"AZ" border="AZ"0"AZ" cellspacing="AZ"2"AZ" cellpadding="AZ"30"AZ" bgcolor="AZ"black"AZ" height="AZ"100%"AZ">" CRLF
               "<tr bgcolor="AZ"#5385bc"AZ" height="AZ"50"AZ">" CRLF
                  "<td height="AZ"50"AZ"><font size="AZ"+3"AZ" color="AZ"white"AZ"><strong>"Kopf_Pendel_"<br>"CRLF Kopf_WHZ_"</strong></font></p>" CRLF
                  "</td>" CRLF
               "</tr>" CRLF
               "<tr valign="AZ"top"AZ" bgcolor="AZ"white"AZ">" CRLF
                  "<td>" CRLF
                     "<table border="AZ"0"AZ" cellspacing="AZ"10"AZ" cellpadding="AZ"3"AZ">" CRLF;

char HTML_Zeile_Anfang[] PROGMEM =
                        "<tr>" CRLF
                           "<td>" CRLF;
char HTML_Zeile_Mitte[] PROGMEM =
                              // Uhrzeit
                           "</td>" CRLF
                           "<td>" CRLF;
char HTML_Zeile_Ende[] PROGMEM =
                           "</td>" CRLF
                        "</tr>" CRLF;


char HTML_Ende[] PROGMEM =
                     "</table>" CRLF
                  "</td>" CRLF
               "</tr>" CRLF
            "</table>" CRLF
// mailto: E-Mail@KarstenDonat.de Hardware
// Stang DB
// Apel PHP
         "</font>" CRLF
      "</body>" CRLF
   "</html>" CRLF;


