//---------------------------------------------------------------------------
// Projekt    : Pendel - Westschsische Hochschule Zwickau
// Datei      : main.cpp
// Datum      : 15.07.2007
// Version    : 1.16
// Autor      : Karsten Donat
// MCU Typ    : atmega168
//---------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "config.h"
#include "LCD_Text.h"
#include "Standard.h"
#include "uart.h"
#include "Uhr.h"
#include "ENC28J60.h"
#include "mem-check.h"

//char Version[] PROGMEM = "1.16"; // im Flash ==> jetzt config
Boolean DCF_Signal=false;

// RS232 Debug (0=keine Debugausgabe)
#define RS232_DEBUG

void init (void)
{
   // Info-LED einschalten whrend Init
   SetBit  (InfoLED_DDR ,InfoLED_PIN); // Info-LED
   ClearBit(InfoLED_PORT,InfoLED_PIN); // LED an (Pin?=0, LED an 5V)

   // Watchdog Timer auf 8s aktivieren
   wdt_enable(WDTO_8S);

   Uhr_Create(false);

   wdt_reset(); // WatchDog Timer zurcksetzen


   /*
    *  Initialize UART library, pass baudrate and AVR cpu clock
    *  with the macro 
    *  UART_BAUD_SELECT() (normal speed mode )
    *  or 
    *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
    */
   uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 

   /*
    * now enable interrupt, since UART library is interrupt controlled
    */
   sei();

   wdt_reset(); // WatchDog Timer zur'fccksetzen

   /*ENC Initialisieren*/
   enc28j60Init();

   while (ENC28J60_Interrupt0==true)
      ENC28J60_Process_Interrupt();


   wdt_reset(); // WatchDog Timer zur'fccksetzen


   LCD_Text_Create();

   wdt_reset(); // WatchDog Timer zur'fccksetzen

   while (ENC28J60_Interrupt0==true)
      ENC28J60_Process_Interrupt();

   // Info-Text anzeigen
   LCD_Clear_display();
   //LCD_Write_String_Flash("Foucault-Pendel  Version ");
   //LCD_Write_String_Flash_p(Version);
   LCD_Write_String_Flash_p(Kopf_Pendel);
   LCD_GotoXY(1,2);
   while (ENC28J60_Interrupt0==true)
      ENC28J60_Process_Interrupt();
   //LCD_Write_String_Flash("Westschsische Hochschule Zwickau 2007");
   LCD_Write_String_Flash_p(Kopf_WHZ);
   // whrend des wartens Ethernetpakete verarbeiten
   for (Byte b=0;b<150;b++)
   {
      wdt_reset(); // WatchDog Timer zur'fccksetzen
      while (ENC28J60_Interrupt0==true)
         ENC28J60_Process_Interrupt();
      delay_ms_grob(20);
   }
   LCD_Clear_display();


   wdt_reset(); // WatchDog Timer zur'fccksetzen

   // PORTD.3 Eingang: DCF Funkempf'e4nger
   ClearBit(DCF77_DDR,DCF77_PIN);
   //EICRA |= (1<<ISC11) + (1<<ISC10); // INT1 Low level
   //EIMSK |= (1<<INT1 ); // INT1 aktiviert

   // Ethernetcontroller Initialisieren
   //ENC28J60_Create();

//    uart_puts("String stored in SRAM\n");
//    uart_puts_P("String stored in FLASH\n");

//   sei();

   LCD_Clear_display();

   // Info-LED ausschalten
   SetBit(InfoLED_PORT,InfoLED_PIN); // LED aus (Pin?=1, LED an 5V)
}

ISR (INT1_vect)
{
   DCF_Signal = true;

//   char Buffer[5];
//   IntToStr(CheckBit(DCF77_PORT,DCF77_PIN), Buffer);
//   uart_putc(CheckBit(DCF77_PORT,DCF77_PIN));
//uart_putc(PORTD);//DCF77_PORT);

/*   IntToStr(PORTD, Buffer);
   uart_puts(Buffer);
   uart_putc(' ');
   DWordToStr(Ticks, Buffer);
   uart_puts(Buffer);
   uart_putc(';');
*/
}



void Update_Anzeige()
{
   char Buffer[5];

   if (Update_Datum==true)
   {
      Update_Datum=false;
      LCD_GotoXY(22,1);

      ItoA_Zeit( Tag, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(".");
      ItoA_Zeit( Monat, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(".");
   
      // vorheriger Status des globalen Interrupt-Flags sichern da Jahr 16-Bit volatile
      //uint8_t sreg_tmp;
      //sreg_tmp = SREG;    /* Sichern */
      //cli();
      IntToStr( Jahr, Buffer);
      //SREG = sreg_tmp;    /* Wiederherstellen */

      LCD_Write_String(Buffer);
   }

   if (Update_Zeit==true)
   {
      Update_Zeit=false;
      // InfoLED blinken im Sekundenrythmus
      InfoLED_PORT ^= (1<<InfoLED_PIN); //toggle PB0 each time this happens

      //Zeit anzeigen
      LCD_GotoXY(33,1);
      ItoA_Zeit( Stunde, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(":");
      ItoA_Zeit( Minute, Buffer);
      LCD_Write_String(Buffer);
      LCD_Write_String(":");
      ItoA_Zeit( Sekunde, Buffer);
      LCD_Write_String(Buffer);
   } //if
}

void RestoreInterrupts(void)
{
   // restore interrupt configuration 
   /*cli();
   TIMSK  = (1<<OCIE1A);
   GICR   = (1<<INT0)|(1<<INT1);
   UCSRB |= (1<<RXCIE);
   SREG  |= iflag;*/
}


int main (void)
{

   // Initialisierungsroutinen aufrufen
   init();

   Word c;

   // Variablen zum Uhr stellen per RS232
   Boolean uart_Datum_Zeit=false;
   Byte    uart_Datum_Zeit_Position=1;


   // IP Adresse anzeigen
   {
      LCD_GotoXY(1,1);
/*      LCD_Write_Byte(config.my_ip[3]);LCD_Write_String(".");
      LCD_Write_Byte(config.my_ip[2]);LCD_Write_String(".");
      LCD_Write_Byte(config.my_ip[1]);LCD_Write_String(".");
      LCD_Write_Byte(config.my_ip[0]);*/
      LCD_Write_Byte(myip[0]);LCD_Write_String(".");
      LCD_Write_Byte(myip[1]);LCD_Write_String(".");
      LCD_Write_Byte(myip[2]);LCD_Write_String(".");
      LCD_Write_Byte(myip[3]);

   }

   
   // Select IDLE sleep Mode
//   MCUCR &= ~((1<<SM1)|(1<<SM0)|(1<<SM2));


   while (true) // Mainloop
   {
      wdt_reset(); // WatchDog Timer zur'fccksetzen


      // Microcontroller schlafen legen //
      // Enable sleep mode
//      MCUCR |= (1<<SE);
      // Call "sleep" assembler instruction
//      __asm__ __volatile__ ("sleep\n\t"::);
      // Disable sleep mode
//      MCUCR &= ~(1<<SE);
      // Microcontroller schlafen legen //

/*
LCD_GotoXY(1,2);
LCD_Write_Word(ENC28J60_Read_Control_Register_8_Bit(MISTAT___MII_Status_Register));

char Buffer[5];
//DWordToStr(ENC28J60_Read_Control_Register_16_Bit(MAMXFLL__Maximum_Frame_Length),Buffer);
DWordToStr(ENC28J60_Read_Control_Register_8_Bit(MISTAT___MII_Status_Register),Buffer);
//DWordToStr(ENC28J60_Read_Control_Register_8_Bit(MAADR1___MAC_Address_Byte_1),Buffer);
      LCD_Write_String(Buffer);
*/

      // Ticks anzeigen
/*      LCD_GotoXY(1,2);
      char Buffer[10];
      DWordToStr(Ticks,Buffer);
      LCD_Write_String(Buffer);
*/

      // UART Interrupt bearbeiten


        /*
         * Get received character from ringbuffer
         * uart_getc() returns in the lower byte the received character and 
         * in the higher byte (bitmask) the last receive error
         * UART_NO_DATA is returned when no data is available.
         *
         */
        c = uart_getc();
        if ( c & UART_NO_DATA )
        {
            /* 
             * no data available from UART 
             */
        }
        else
        {
            /*
             * new data available from UART
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )
            {
                /* Framing Error detected, i.e no stop bit detected */
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )
            {
                /* 
                 * Overrun, a character already present in the UART UDR register was 
                 * not read by the interrupt handler before the next character arrived,
                 * one or more received characters have been dropped
                 */
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )
            {
                /* 
                 * We are not reading the receive buffer fast enough,
                 * one or more received character have been dropped 
                 */
                uart_puts_P("Buffer overflow error: ");
            }
            /* 
             * send received character back
             */
            uart_putc( (unsigned char)c );

            // Uhrzeit ber RS232 stellen
            // 255 senden, danach jeweils als 1 Byte...  TagMonatJahrStundeMinuteSekunden
            // das Jahr kann von 0 = 2000 bis 254 = 2254 gesendet werden
            if (c==255)
            {
               uart_Datum_Zeit=true;
               uart_Datum_Zeit_Position=1;
            }
            else
            if (uart_Datum_Zeit==true)
            {
               if (uart_Datum_Zeit_Position==1)
               {
                  Tag=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==2)
               {
                  Monat=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==3)
               {
                  Jahr=2000+((unsigned char)c);
               }
               else
               if (uart_Datum_Zeit_Position==4)
               {
                  Stunde=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==5)
               {
                  Minute=(unsigned char)c;
               }
               else
               if (uart_Datum_Zeit_Position==6)
               {
                  Sekunde=(unsigned char)c;
                  uart_Datum_Zeit=false;
                  Update_Datum=true;
               }
               else
               {
                  uart_Datum_Zeit=false;
               }

               uart_Datum_Zeit_Position++;
            }
        }

      // ENC28J60 Interrupt bearbeiten
      if (ENC28J60_Interrupt0==true)
      {
         ENC28J60_Process_Interrupt();
         /*uart_puts("Interrupt ");
         char Buffer[10];
         DWordToStr(Ticks,Buffer);
         uart_puts(Buffer);
         uart_putc(0x0D);
         uart_putc(0x0A);*/

         /*LCD_GotoXY(1,2);
         LCD_Write_String("Interrupt ");
         char Buffer[10];
         DWordToStr(Ticks,Buffer);
         LCD_Write_String(Buffer);*/
      }
         ENC28J60_Process_Interrupt();




      // DCF_77 Uhr
      if (DCF_Signal == true)
      {
         DCF_Signal = false;
/*         LCD_GotoXY(1,2);
         LCD_Write_String("INT1 ");
         char Buffer[5];
         DWordToStr( Ticks, Buffer);
         LCD_Write_String(Buffer);

         LCD_Write_String(" Bit= ");
         IntToStr(CheckBit(DCF77_PORT,DCF77_PIN), Buffer);
         LCD_Write_String(Buffer);

         
         InfoLED_PORT ^= (CheckBit(DCF77_PORT,DCF77_PIN)<<InfoLED_PIN);*/
      }


      Update_Anzeige();

      // freien RAM anzeigen
//      LCD_GotoXY(1,2);
//      LCD_Write_Word(momentan_frei_);
   }

   return 0;
}

